"""
League Repository Tests.
"""

from assertpy import assert_that
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from football_data.models import League
from football_data.repositories import LeagueRepository


def create_maker() -> sessionmaker:
    """
    Creates the Session Maker.
    :return: sessionmaker
    """
    engine = create_engine('sqlite://')
    League.metadata.create_all(bind=engine)
    return sessionmaker(bind=engine, expire_on_commit=False)


def test_league_exists():
    """
    Tests if a League exists.
    """
    maker = create_maker()
    league = League(id=1, code='NFC', description='National Football Conference')
    repo = LeagueRepository(maker)
    repo.save(league)

    result = repo.league_exits(league)
    assert_that(result).is_true()


def test_league_does_not_exist():
    """
    Tests if a league does not exist.
    """
    maker = create_maker()
    league = League(id=1, code='NFC', description='National Football Conference')
    league2 = League(id=2, code='AFC', description='American Football Conference')
    repo = LeagueRepository(maker)
    repo.save(league)

    result = repo.league_exits(league2)

    assert_that(result).is_false()


def test_get_league():
    """
    Tests retrieving a League by id
    """
    maker = create_maker()
    league = League(id=1, code='NFC', description='National Football Conference')
    league2 = League(id=2, code='AFC', description='American Football Conference')
    repo = LeagueRepository(maker)
    repo.save(league)
    repo.save(league2)

    result = repo.get_league(id=1)

    assert_that(result).is_equal_to(league)


def test_get_league_by_code():
    """
    Tests retrieving a League by Code
    """

    maker = create_maker()
    league = League(id=1, code='NFC', description='National Football Conference')
    league2 = League(id=2, code='AFC', description='American Football Conference')
    repo = LeagueRepository(maker)
    repo.save(league)
    repo.save(league2)

    result = repo.get_league(code='AFC')
    assert_that(result).is_equal_to(league2)


def test_get_league_not_exists():
    """
    Tests retrieving a League that does not exist.
    """
    maker = create_maker()
    league = League(id=1, code='NFC', description='National Football Conference')
    league2 = League(id=2, code='AFC', description='American Football Conference')
    repo = LeagueRepository(maker)
    repo.save(league)
    repo.save(league2)

    result = repo.get_league(code='JJJ')
    assert_that(result).is_none()


def test_get_league_no_args():
    """
    Tests retrieving a league without Args.
    """
    maker = create_maker()
    league = League(id=1, code='NFC', description='National Football Conference')
    league2 = League(id=2, code='AFC', description='American Football Conference')
    repo = LeagueRepository(maker)
    repo.save(league)
    repo.save(league2)

    result = repo.get_league()
    assert_that(result).is_none()


def test_get_leagues():
    """
    Tests retrieving all Leagues
    """
    maker = create_maker()
    league = League(id=1, code='NFC', description='National Football Conference')
    league2 = League(id=2, code='AFC', description='American Football Conference')
    repo = LeagueRepository(maker)
    repo.save(league)
    repo.save(league2)

    result = repo.get_leagues()
    assert_that(result).contains_only(league, league2)



"""
Tests for the Statistics Repository.
"""

from assertpy import assert_that
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from football_data.models import Statistic
from football_data.repositories import StatisticRepository


def create_maker() -> sessionmaker:
    """
    Creates the Sqlite Database Engine
    :return: sessionmaker
    """
    engine = create_engine('sqlite://')
    Statistic.metadata.create_all(bind=engine)
    return sessionmaker(bind=engine, expire_on_commit=False)


def test_player_statistic_exists():
    """
    Tests if a Statistic exists.
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, player_id=1, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)

    result = repo.statistic_exists(stat)
    assert_that(result).is_true()


def test_team_statistic_exists():
    """
    Tests if a Team Statistic Exists.
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, team_id=1, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)

    result = repo.statistic_exists(stat)
    assert_that(result).is_true()


def test_statistic_does_not_exist():
    """
    Tests that a statistic does not exist
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, team_id=1, schedule_id=1, value=20, category_id=1)
    stat2 = Statistic(id=2, statistic_code_id=1, team_id=2, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)

    result = repo.statistic_exists(stat2)
    assert_that(result).is_false()


def test_get_statistic():
    """
    Tests retrieving a Statistic by the ID Value.
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, team_id=1, schedule_id=1, value=20, category_id=1)
    stat2 = Statistic(id=2, statistic_code_id=1, team_id=2, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)
    repo.save(stat2)

    result = repo.get_statistic(1)
    assert_that(result).is_not_none().is_equal_to(stat)


def test_get_statistic_does_not_exist():
    """
    Tests retrieving Statistic that does not exist.
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, team_id=1, schedule_id=1, value=20, category_id=1)
    stat2 = Statistic(id=2, statistic_code_id=1, team_id=2, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)
    repo.save(stat2)

    result = repo.get_statistic(3)
    assert_that(result).is_none()


def test_get_statistics_by_team_id():
    """
    Tests retrieving the statistics by Team ID
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, player_id=1, schedule_id=1, value=20, category_id=1)
    stat2 = Statistic(id=2, statistic_code_id=1, team_id=2, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)
    repo.save(stat2)

    result = repo.get_statistics(team_id=2)
    assert_that(result).contains_only(stat2)


def test_get_statistics_by_player_id():
    """
    Tests retrieving the statistics by Player ID
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, player_id=1, schedule_id=1, value=20, category_id=1)
    stat2 = Statistic(id=2, statistic_code_id=1, team_id=2, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)
    repo.save(stat2)

    result = repo.get_statistics(player_id=1)
    assert_that(result).contains_only(stat)


def test_get_statistics_by_schedule_id():
    """
    Tests retrieving the statistics by Schedule ID
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, player_id=1, schedule_id=1, value=20, category_id=1)
    stat2 = Statistic(id=2, statistic_code_id=1, team_id=2, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)
    repo.save(stat2)

    result = repo.get_statistics(schedule_id=1)
    assert_that(result).contains_only(stat, stat2)


def test_get_statistics_by_player_team_and_schedule():
    """
    Tests retrieving the statistics by Player, Team, and Schedule.
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, team_id=1, player_id=1, schedule_id=1, value=20,
                     category_id=1)
    stat2 = Statistic(id=2, statistic_code_id=1, team_id=2, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)
    repo.save(stat2)

    result = repo.get_statistics(player_id=1, team_id=1, schedule_id=1)
    assert_that(result).contains_only(stat)


def test_get_statistics_by_player_team():
    """
    Tests retrieving statistics by player and team
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, player_id=1, team_id=1, schedule_id=1, value=20,
                     category_id=1)
    stat2 = Statistic(id=2, statistic_code_id=1, team_id=2, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)
    repo.save(stat2)

    result = repo.get_statistics(player_id=1, team_id=1)
    assert_that(result).contains_only(stat)


def test_get_statistics_by_player_schedule():
    """
    Tests retrieving statistics by player and schedule.
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, player_id=1, schedule_id=1, value=20, category_id=1)
    stat2 = Statistic(id=2, statistic_code_id=1, team_id=2, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)
    repo.save(stat2)

    result = repo.get_statistics(player_id=1, schedule_id=1)
    assert_that(result).contains_only(stat)


def test_get_statistics_by_team_schedule():
    """
    Tests retrieving statistics by team and schedule.
    """
    maker = create_maker()
    stat = Statistic(id=1, statistic_code_id=1, team_id=1, schedule_id=1, value=20, category_id=1)
    stat2 = Statistic(id=2, statistic_code_id=1, team_id=2, schedule_id=1, value=20, category_id=1)
    repo = StatisticRepository(maker)
    repo.save(stat)
    repo.save(stat2)

    result = repo.get_statistics(team_id=2, schdule_id=1)
    assert_that(result).contains_only(stat2)



"""
Tests for the Statistic Category Code Repository.
"""

from assertpy import assert_that
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from football_data.models import StatisticCategory
from football_data.repositories import StatisticCategoryRepository


def create_maker():
    """
    Creates the In Memory Database.
    """
    engine = create_engine('sqlite://')
    StatisticCategory.metadata.create_all(bind=engine)
    return sessionmaker(bind=engine, expire_on_commit=False)


def test_get_statistic_category_code():
    """
    Tests retrieving a Statistic Category Code.
    """
    maker = create_maker()
    code = StatisticCategory(id=1, code='O', description='Offense')
    repo = StatisticCategoryRepository(maker)
    repo.save(code)

    result = repo.get_statistic_category(code='O')
    assert_that(result).is_not_none().is_equal_to(code)


def test_get_statistic_category_code_by_id():
    """
    Tests retrieving the statistic category by primary key ID.
    """
    maker = create_maker()
    code = StatisticCategory(id=1, code='O', description='Offense')
    repo = StatisticCategoryRepository(maker)
    repo.save(code)

    result = repo.get_statistic_category(id=1)
    assert_that(result).is_not_none().is_equal_to(code)


def test_get_statistic_category_not_exist():
    """
    Tests retrieving a Statistic Category that does not exist.
    """
    maker = create_maker()
    code = StatisticCategory(id=1, code='O', description='Offense')
    repo = StatisticCategoryRepository(maker)
    repo.save(code)

    result = repo.get_statistic_category(code='D')
    assert_that(result).is_none()


def test_get_statistic_category_no_args():
    """
    Tests retrieving a statistic category without args.
    """
    maker = create_maker()
    code = StatisticCategory(id=1, code='O', description='Offense')
    repo = StatisticCategoryRepository(maker)
    repo.save(code)

    result = repo.get_statistic_category()
    assert_that(result).is_none()


def test_statistic_category_exists():
    """
    Tests a statistic category exists.
    """

    maker = create_maker()
    code = StatisticCategory(id=1, code='O', description='Offense')
    repo = StatisticCategoryRepository(maker)
    repo.save(code)

    result = repo.statistic_category_exists(code)
    assert_that(result).is_true()


def test_statistic_category_does_not_exist():
    """
    Tests that a statistic category does not exist.
    """
    maker = create_maker()
    code = StatisticCategory(id=1, code='O', description='Offense')
    code2 = StatisticCategory(id=2, code='D', description='Defense')
    repo = StatisticCategoryRepository(maker)
    repo.save(code)

    result = repo.statistic_category_exists(code2)
    assert_that(result).is_false()


def test_get_statistic_categories():
    """
    Tests retrieving all Statistic Category Codes.
    """
    maker = create_maker()
    code = StatisticCategory(id=1, code='O', description='Offense')
    code2 = StatisticCategory(id=2, code='D', description='Defense')
    repo = StatisticCategoryRepository(maker)
    repo.save(code)
    repo.save(code2)

    result = repo.get_statistic_categories()
    assert_that(result).is_not_empty().contains_only(code, code2)



"""
Tests for the Player Repository.
"""

from assertpy import assert_that
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from football_data.models import Player, Position
from football_data.repositories import PlayerRepository


def create_maker():
    """
    Creates the SQL Alchemy Session Maker.
    """

    engine = create_engine('sqlite://')
    Player.metadata.create_all(bind=engine)
    return sessionmaker(bind=engine, expire_on_commit=False)


def test_player_exists():
    """
    Tests if a player exists.
    """
    maker = create_maker()
    player = Player(id=1, url="www.google.com", name="Jim Smith")
    repo = PlayerRepository(maker)
    repo.save(player)

    result = repo.player_exits(player)
    assert_that(result).is_true()


def test_player_does_not_exist():
    """
    Tests if a player does not exist.
    """
    maker = create_maker()
    player = Player(id=1, url="www.google.com", name="Jim Smith")
    player2 = Player(id=2, url="www.player.com", name="Bill Smith")
    repo = PlayerRepository(maker)
    repo.save(player)

    result = repo.player_exits(player2)
    assert_that(result).is_false()


def test_get_player_by_id():
    """
    Tests retrieving the Player by Id
    """
    maker = create_maker()
    player = Player(id=1, url="www.google.com", name="Jim Smith")
    repo = PlayerRepository(maker)
    repo.save(player)

    result = repo.get_player(id=1)
    assert_that(result).is_not_none().is_equal_to(player)


def test_get_player_by_url():
    """
    Tests retrieving Player by Url.
    """
    maker = create_maker()
    player = Player(id=1, url="www.google.com", name="Jim Smith")
    repo = PlayerRepository(maker)
    repo.save(player)

    result = repo.get_player(url='www.google.com')
    assert_that(result).is_not_none().is_equal_to(player)


def test_get_player_not_exists():
    """
    Tests retrieving a Player that is not present.
    """
    maker = create_maker()
    player = Player(id=1, url="www.google.com", name="Jim Smith")
    repo = PlayerRepository(maker)
    repo.save(player)

    result = repo.get_player(url='www.player.com')
    assert_that(result).is_none()


def test_get_players():
    """
    Tests retrieving all players.
    """
    maker = create_maker()
    player = Player(id=1, url="www.google.com", name="Jim Smith")
    repo = PlayerRepository(maker)
    repo.save(player)

    result = repo.get_players()
    assert_that(result).is_not_empty().contains(player)


def test_get_players_by_position_id():
    """
    Tests retrieving a player by position id
    """
    maker = create_maker()

    position = Position(id=1, code='QB', description='Quarterback')
    player = Player(id=1, url="www.google.com", name="Jim Smith", position_id=1)
    repo = PlayerRepository(maker)
    repo.save_all([player, position])

    result = repo.get_players(position_id=1)
    assert_that(result).is_not_none().contains(player)


def test_get_players_by_position_code():
    """
    Tests retrieving Players by Position Code.
    """
    maker = create_maker()

    position = Position(id=1, code='QB', description='Quarterback')
    player = Player(id=1, url="www.google.com", name="Jim Smith", position_id=1)
    repo = PlayerRepository(maker)
    repo.save_all([player, position])

    result = repo.get_players(position_code='QB')
    assert_that(result).is_not_empty().contains(player)



"""
Football Data Models.
"""

from typing import Optional

from sqlalchemy.orm import DeclarativeBase, MappedAsDataclass, Mapped, mapped_column
from sqlalchemy.types import BigInteger, String, Integer, REAL


class Base(MappedAsDataclass, DeclarativeBase):
    """
    Base Class for Data Models.
    """


class Player(Base):
    """
    Player Data Model Class
    """

    __tablename__ = 'players'

    url: Mapped[str] = mapped_column(String(255))
    name: Mapped[str] = mapped_column(String(500))
    position_id: Mapped[Optional[int]] = mapped_column(BigInteger, default=None)
    id: Mapped[Optional[int]] = mapped_column(BigInteger().with_variant(Integer, 'sqlite'),
                                              primary_key=True, autoincrement=True,
                                              default=None)


class Position(Base):
    """
    Position Code Model Class.
    """

    __tablename__ = 'position_codes'

    code: Mapped[str] = mapped_column(String(5))
    description: Mapped[str] = mapped_column(String(50))
    id: Mapped[Optional[int]] = mapped_column(Integer, primary_key=True, autoincrement=True,
                                              default=None)


class Schedule(Base):
    """
    Schedule Item Model Class.
    """

    __tablename__ = 'schedule'

    team_id: Mapped[int]
    opponent_id: Mapped[int]
    year_value: Mapped[int]
    week_number: Mapped[int]
    game_id: Mapped[int] = mapped_column(BigInteger)
    url: Mapped[str] = mapped_column(String(255))
    type_id: Mapped[int]
    is_home: Mapped[bool]
    id: Mapped[Optional[int]] = mapped_column(BigInteger().with_variant(Integer, 'sqlite'),
                                              primary_key=True, autoincrement=True,
                                              default=None)


class StatisticCategory(Base):
    """
    Statistic Category Code Model.
    """

    __tablename__ = 'statistic_categories'

    code: Mapped[str] = mapped_column(String(10))
    description: Mapped[str] = mapped_column(String(50))
    id: Mapped[Optional[int]] = mapped_column(Integer, primary_key=True, autoincrement=True,
                                              default=None)


class StatisticCode(Base):
    """
    Statistic Code Model.
    """

    __tablename__ = 'statistic_codes'

    code: Mapped[str] = mapped_column(String(15))
    description: Mapped[str] = mapped_column(String(100))
    grouping: Mapped[Optional[str]] = mapped_column(String(100), default=None)
    id: Mapped[Optional[int]] = mapped_column(BigInteger().with_variant(Integer, 'sqlite'),
                                              primary_key=True, autoincrement=True,
                                              default=None)


class Statistic(Base):
    """
    Statistic Code Model.
    """

    __tablename__ = 'statistics'

    statistic_code_id: Mapped[int] = mapped_column(BigInteger)
    schedule_id: Mapped[int] = mapped_column(BigInteger)
    value: Mapped[float] = mapped_column(REAL)
    category_id: Mapped[int]
    player_id: Mapped[Optional[int]] = mapped_column(BigInteger, default=None)
    team_id: Mapped[Optional[int]] = mapped_column(Integer, default=None)
    id: Mapped[Optional[int]] = mapped_column(BigInteger().with_variant(Integer, 'sqlite'),
                                              primary_key=True, autoincrement=True,
                                              default=None)


class Team(Base):
    """
    Team Data Model.
    """

    __tablename__ = 'team'

    url: Mapped[str] = mapped_column(String(255))
    code: Mapped[str] = mapped_column(String(5))
    name: Mapped[str] = mapped_column(String(100))
    id: Mapped[Optional[int]] = mapped_column(Integer, primary_key=True, autoincrement=True,
                                              default=None)


class TypeCode(Base):
    """
    Type Code Data Model
    """

    __tablename__ = 'type_codes'

    code: Mapped[str] = mapped_column(String(10))
    description: Mapped[str] = mapped_column(String(50))
    id: Mapped[Optional[int]] = mapped_column(Integer, primary_key=True, autoincrement=True,
                                              default=None)


class TeamStaff(Base):
    """
    Data Model for the Team Staff relationship.
    """

    __tablename__ = 'team_staff'

    player_id: Mapped[int] = mapped_column(BigInteger)
    team_id: Mapped[int] = mapped_column(Integer)
    year_value: Mapped[int] = mapped_column(Integer)
    id: Mapped[Optional[int]] = mapped_column(BigInteger().with_variant(Integer, 'sqlite'),
                                              primary_key=True, autoincrement=True,
                                              default=None)


class League(Base):
    """
    Data Model for the League.
    """

    __tablename__ = 'leages'

    code: Mapped[str] = mapped_column(String(10))
    description: Mapped[str] = mapped_column(String(100))
    id: Mapped[Optional[int]] = mapped_column(Integer, primary_key=True, default=None)


class TeamLeague(Base):
    """
    Data Model for the League and Team Relationship table.
    """

    __tablename__ = 'team_leagues'

    team_id: Mapped[int] = mapped_column(Integer)
    league_id: Mapped[int] = mapped_column(Integer)
    year_value: Mapped[int] = mapped_column(Integer)
    id: Mapped[Optional[int]] = mapped_column(BigInteger().with_variant(Integer, 'sqlite'),
                                              primary_key=True, autoincrement=True,
                                              default=None)
